@@ -1,12 +1,11 @@
-/* Extracted from the Himeno Benchmark */
-
 float gosa, ss, s0, omega;
 float a[4][L][M][N], b[3][L][M][N], c[3][L][M][N];
 float p[L][M][N], bnd[L][M][N], wrk1[L][M][N], wrk2[L][M][N];
 
-for(int i=1 ; i<L; i++)
-  for(int j=1 ; j<M ; j++)
-    for(int k=1 ; k<N ; k++){
+#pragma omp parallel for private(s0,ss) reduction(+:gosa) schedule(static)
+for(int i=1 ; i<L-1; i++) {
+  for(int j=1 ; j<M-1; j++) {
+    for(int k=1 ; k<N-1; k++){
           s0 = a[0][i][j][k]*p[i+1][j][k]
              + a[1][i][j][k]*p[i][j+1][k]
              + a[2][i][j][k]*p[i][j][k+1]
@@ -29,3 +28,6 @@ for(int i=1 ; i<L; i++)
           gosa += ss*ss;
           wrk2[i][j][k] = p[i][j][k] + omega*ss;
 }
+}
+}
+
