@@ -1,5 +1,11 @@
+#ifndef _KERNCRAFT_H
+#define _KERNCRAFT_H
+#include <stddef.h>
 #include <stdlib.h>
 
+#define swap(a, b) {void* t = a; a = b; b = t;}
+
+
 inline void* aligned_malloc(size_t size, size_t align) {
     // Based on http://stackoverflow.com/q/16376942
     void *result;
@@ -11,4 +17,6 @@ inline void* aligned_malloc(size_t size, size_t align) {
          if(posix_memalign(&result, align, size)) result = 0;
     #endif
     return result;
-}
\ No newline at end of file
+};
+
+#endif
