@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 import unittest
 import sys
 
@@ -9,9 +9,12 @@ suite = unittest.TestLoader().loadTestsFromNames(
         'test_intervals',
         'test_kernel',
         'test_layer_condition',
-        'test_likwid_bench_auto'
+        'test_incore_model',
+        'test_cacheprediction',
+        'test_machinemodel',
+        'test_example_files',
     ]
 )
 
-testresult = unittest.TextTestRunner(verbosity=2).run(suite)
+testresult = unittest.TextTestRunner(verbosity=2, buffer=True).run(suite)
 sys.exit(0 if testresult.wasSuccessful() else 1)
